<?php
declare(strict_types=1);

namespace Inti\Wp\Helpers;

use InvalidArgumentException;
use RuntimeException;
use function array_flip;
use function count;
use function floor;
use function get_option;
use function intval;
use function update_option;

/**
 * Class IdOffsetter
 *
 * @package Inti\Wp\Helpers
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp-helpers
 */
final class IdOffsetter
{
    const BASE_OFFSET = 1e7;

    private string $optionName;
    /** @var array<string,int> */
    private array $offsets;

    public function __construct(string $optionName)
    {
        $this->optionName = $optionName;
        $this->offsets = get_option($this->optionName, []);
    }

    public function offset(int $id, string $type): int
    {
        if (!isset($this->offsets[$type])) {
            $this->register($type);
        }

        return intval($this->offsets[$type] + $id);
    }

    public function getType(int $id): string
    {
        if ($id < self::BASE_OFFSET) {
            throw new InvalidArgumentException(sprintf(
                '%s: The given id %d isn\'t offset.',
                __METHOD__, $id
            ));
        }

        $offset = intval(floor($id / self::BASE_OFFSET) * self::BASE_OFFSET);
        $offsets = array_flip($this->offsets);

        if (!isset($offsets[$offset])) {
            throw new RuntimeException(sprintf(
                '%s: Offset %d isn\t registered.',
                __METHOD__, $offset
            ));
        }

        return $offsets[$offset];
    }

    public function unoffset(int $id): int
    {
        $type = $this->getType($id);
        return intval($id - $this->offsets[$type]);
    }

    public function isOffset(int $id): bool
    {
        return $id > self::BASE_OFFSET;
    }

    private function register(string $type): void
    {
        if (!isset($this->offsets[$type])) {
            $this->offsets[$type] = intval((count($this->offsets) + 1) * self::BASE_OFFSET);
            update_option($this->optionName, $this->offsets);
        }
    }
}
