<?php
declare(strict_types=1);

namespace Inti\Wp\Helpers\Provider;

use function class_implements;
use function in_array;
use function is_string;

/**
 * Class ProviderList
 *
 * @package Inti\Wp\Helpers\Provider
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp-helpers
 */
final class ProviderList implements ProviderListInterface
{
    private ProviderManager $manager;
    private array $providers;

    public function __construct(ProviderManager $manager, array $providers)
    {
        $this->manager = $manager;
        $this->providers = $providers;
    }

    public function getProvider(string $type, ?string $interface = null): ?ProviderInterface
    {
        foreach ($this->getProviders($interface) as &$provider) {
            if ($provider->getType() === $type) {
                return $provider;
            }
        }

        return null;
    }

    /**
     * @param string|null $interface
     * @return ProviderInterface[]
     */
    public function getProviders(?string $interface = null): array
    {
        foreach ($this->providers as &$provider) {
            if (is_string($provider)) {
                $provider = $this->manager->get($provider);
            }

            if (isset($interface) && in_array($interface, class_implements($provider))) {
                $providers[$interface] = $provider;
            }
        }

        return isset($interface)
            ? ($providers[$interface] ?? [])
            : $this->providers;
    }
}
